@@ -33,6 +33,7 @@ import torch
 from PIL import Image
 from torch.utils.tensorboard import SummaryWriter
 from pytorch_lightning.utilities import rank_zero_only
+from pytorch_segmentation_models_trainer.tools.visualization.crossfield_plot import get_tensorboard_image_seg_display
 
 logging.getLogger('matplotlib').setLevel(level=logging.CRITICAL)
 
@@ -53,6 +54,37 @@ def denormalize_np_array(image: np.ndarray, \
 
     return image * np.array(mean)[..., None, None] + np.array(mean)[..., None, None]
 
+def batch_denormalize_tensor(tensor, mean=None, std=None, inplace=False):
+    """Denormalize a batched tensor image with batched mean and batched standard deviation.
+    .. note::
+        This transform acts out of place by default, i.e., it does not mutates the input tensor.
+    Args:
+        tensor (Tensor): Tensor image of size (B, C, H, W) to be normalized.
+        mean (sequence): Tensor means of size (B, C).
+        std (sequence): Tensor standard deviations of size (B, C).
+        inplace(bool,optional): Bool to make this operation inplace.
+    Returns:
+        Tensor: Normalized Tensor image.
+    """
+    mean = mean if mean is not None else torch.tensor([0.485, 0.456, 0.406]).expand(
+        tensor.shape[0], tensor.shape[1])
+    std = std if std is not None else torch.tensor([0.229, 0.224, 0.225]).expand(
+        tensor.shape[0], tensor.shape[1])
+    assert len(tensor.shape) == 4, \
+        "tensor should have 4 dims (B, H, W, C) , not {}".format(len(tensor.shape))
+    assert len(mean.shape) == len(std.shape) == 2, \
+        "mean and std should have 2 dims (B, C) , not {} and {}".format(len(mean.shape), len(std.shape))
+    assert tensor.shape[1] == mean.shape[1] == std.shape[1], \
+        "tensor, mean and std should have the same number of channels, not {}, {} and {}".format( tensor.shape[-1], mean.shape[-1], std.shape[-1])
+
+    if not inplace:
+        tensor = tensor.clone()
+
+    mean = mean.to(tensor.dtype)
+    std = std.to(tensor.dtype)
+    tensor.mul_(std[..., None, None]).add_(mean[..., None, None])
+    return tensor
+
 class ImageSegmentationResultCallback(pl.callbacks.base.Callback):
 
     def __init__(self, n_samples: int, output_path: str = None, \
@@ -155,3 +187,18 @@ class ImageSegmentationResultCallback(pl.callbacks.base.Callback):
                 )
             plt.close(fig)
         return
+
+class FrameFieldResultCallback(pl.callbacks.base.Callback):
+
+    @rank_zero_only
+    def on_validation_end(self, trainer, pl_module):
+        val_ds = pl_module.val_dataloader()
+        batch = next(iter(val_ds))
+        image_display = batch_denormalize_tensor(batch["image"]).to(pl_module.device)
+        pred =  pl_module(batch["image"].to(pl_module.device))
+        if "seg" in pred:
+            crossfield = pred["crossfield"] if "crossfield" in pred else None
+            image_seg_display = get_tensorboard_image_seg_display(
+                image_display, pred["seg"], crossfield=crossfield)
+            trainer.logger.experiment.add_images(
+                'seg', image_seg_display, trainer.current_epoch)
